/////////////////////////////////////////////////////////////////////////////
// Decrypt.cpp
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2001 Cricri  All rights reserved.
/////////////////////////////////////////////////////////////////////////////
//
//	This file is subject to the terms of the GNU General Public License as
//	published by the Free Software Foundation.  A copy of this license is
//	included with this software distribution in the file COPYING.  If you
//	do not have a copy, you may obtain a copy by writing to the Free
//	Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
//	This software is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details
/////////////////////////////////////////////////////////////////////////////
//
// This software was based on More FTC & Azertix 9c Those portions are
// Copyright Gloup & Azertix (C) 1999/2000 
//
/////////////////////////////////////////////////////////////////////////////

#include "CriCrack.h"

#define	SUM_MIN3	600

extern QuenotteTV_VideoPlugin	sVP;
extern FTC_CRACK				sCtx;
extern int						ntShift[];

// BYTE	ucClrVal[] = { 0x84, 0x40, 0xFF};

// extern int RAItbl3[];

// static int		nKeyIdx = 0, nI = 0;
// static bool		O287, E287;
static int		ntTable[3456];
static int		nKeyIdx = 0;
static BYTE		ucDelayTable[] = { 0, 2, 2, 0, 1, 0, 2, 1 }; // XYZ decision table


/* ----------------------------- proto ------------------------------------- */

//static int dec_choose( void);

//static int dec_search_lum_cut( void);

//static int dec_search_by_contrast( void);

//static int dec_search_by_lum_rep( void);

//static int dec_search_by_lum_cut( void);

/* ----------------------------- proto extern ------------------------------ */

// inline bool dec_key_init( YUY_T *const pLine);
// void dec_key( const int y, const int nShiftIdx, int *const pntTable);

/* ------------------------------------------------------------------------- */




/* ------------------------------------------------------------------------- */
int dec_cut_lum( YUY_T *const pSrce,
				  const int nPix,
				  const int nStep,
				  const int nLevel)
/* ------------------------------------------------------------------------- */
{
	static int		x;


	if ( pSrce == NULL)
		return(NOK);

	for ( x = 0; x < nPix; x += nStep) {

		if ( pSrce[x].yX > nLevel)
			break;
	}

	return( ( x == nPix) ? NOK : x);
}




/* ------------------------------------------------------------------------- */
inline static int dec_max_lum( YUY_T *const pSrce, const int nPix)
/* ------------------------------------------------------------------------- */
{
	static int		nMax;
	static YUY_T	*p, *pEnd;


	if ( pSrce == NULL)
		return(NOK);

	for ( p = pSrce, nMax = 0, pEnd = pSrce + nPix; p < pEnd; p++) {

		if ( p->yX > nMax)
			nMax = p->yX;
	}

	return( nMax);
}




/* ------------------------------------------------------------------------- */
int dec_sum_lum1( YUY_T *const pSrce, const int nPix)
/* ------------------------------------------------------------------------- */
{
	int		            nSum = 0;
	static  YUY_T	    *p, *pEnd;
    static  __int64     *pN, *pEndN;


	if ( pSrce == NULL)
		return(NOK);

    for ( pN = (__int64 *)pSrce, pEndN = pN + (nPix >> 2); pN < pEndN; pN++) {
	    nSum += (int)(((*pN & 0x00FF000000000000) >> 48)
                    + ((*pN & 0x000000FF00000000) >> 32)
                    + ((*pN & 0x0000000000FF0000) >> 16)
                    +  (*pN & 0x00000000000000FF));
	}

    if ( nPix & 3) {
        for ( p = (YUY_T *)pN, pEnd = pSrce + nPix; p < pEnd; p++) {
		    nSum += p->yX;
	    }
    }
    
	return( nSum);
}



/* ------------------------------------------------------------------------- */
int dec_sum_lum( YUY_T *const pSrce, const int nPix)
/* ------------------------------------------------------------------------- */
{
	static int		nSum;
	static YUY_T	*p, *pEnd;


	if ( pSrce == NULL)
		return(NOK);

	for ( p = pSrce, nSum = 0, pEnd = pSrce + nPix; p < pEnd; p++) {
		nSum += p->yX;
	}

	return( nSum);
}



/* ------------------------------------------------------------------------- */
void dec_shift_cr( __int64 *const pLine, const int nShift)
/* ------------------------------------------------------------------------- */
{
	static __int64	*p;
    static  int     nNbrPixel;


    if ( pLine == NULL)
		return;
	
    /* Be carefull, this routine suppose that the buffer is bigger than the
        maximum X resolution and that nShift value is necessary odd
    */
    nNbrPixel = sVP.CurrentX - nShift + 4 - (nNbrPixel & 3);

	for ( p = pLine + (nNbrPixel >> 2); p > pLine; p--) {
		*p = *p & 0x00FF00FF00FF00FF
             | *(__int64 *)((short *)p - 1) & 0xFF00FF00FF00FF00;
	}
}



/* ------------------------------------------------------------------------- */
int dec_mark( short *const pDest, const int nShift, const int nVal)
/* ------------------------------------------------------------------------- */
{

	if ( pDest == NULL)
		return(NOK);

	/* complete end of line	*/
	for ( int x = sVP.CurrentX - nShift; x < sVP.CurrentX; x++) {
		pDest[x] = (short)nVal;
	}
	
	return(OK);

}




/* ------------------------------------------------------------------------- */
static int dec_search_max_start( YUY_T *const psLine)
/* ------------------------------------------------------------------------- */
{
	static int	nShiftIdx;


	if ( psLine == NULL)
		return(NOK);

	nShiftIdx = NOK;

	if ( dec_max_lum( psLine, ntShift[1]) > sCtx.nLumLevel)
		nShiftIdx = 0;
	else if ( dec_max_lum( &psLine[ntShift[1]], ntShift[1]) > sCtx.nLumLevel)
		nShiftIdx = 1;
	else if ( dec_max_lum( &psLine[ntShift[2]], ntShift[1]) > sCtx.nLumLevel)
		nShiftIdx = 2;

	return( nShiftIdx);

}




/* ------------------------------------------------------------------------- */
static int dec_search_start_cor( YUY_T *const psLine, YUY_T *const psRefLine)
/* ------------------------------------------------------------------------- */
{
	static int		nDiff0, nDiff1, nDiff2, nShiftIdx, x;
	static u_long	nSum;
//	static int		ntSum[0], ntSum[1], ntSum[2];

	

	if ( psLine == NULL || psRefLine == NULL)
		return(NOK);

	sCtx.ntStartCrl[0] = sCtx.ntStartCrl[1] = sCtx.ntStartCrl[2] = 0;

	for ( x = 0; x < ntShift[1]; x++) {

//		nDiff0 = psRefLine[x + ntShift[nLastShiftIdx]].yX - psCurLine[x + ntShift[0]].yX;
		nDiff0 = psRefLine[x].yX - psLine[x].yX;
		nDiff1 = psRefLine[x].yX - psLine[x + ntShift[1]].yX;
		nDiff2 = psRefLine[x].yX - psLine[x + ntShift[2]].yX;
				
		sCtx.ntStartCrl[0] += nDiff0 * nDiff0;
		sCtx.ntStartCrl[1] += nDiff1 * nDiff1;
		sCtx.ntStartCrl[2] += nDiff2 * nDiff2;
	}

	if ( sCtx.ntStartCrl[0] < sCtx.ntStartCrl[1])
		nShiftIdx = ( sCtx.ntStartCrl[2] < sCtx.ntStartCrl[0]) ? 2 : 0;
	else
		nShiftIdx = ( sCtx.ntStartCrl[1] < sCtx.ntStartCrl[2]) ? 1 : 2;

	/* test on validity	*/
	nSum = (sCtx.ntStartCrl[0] + sCtx.ntStartCrl[1] + sCtx.ntStartCrl[2]);
	if ( (sCtx.ntStartCrl[nShiftIdx] << 2) > nSum)
		nShiftIdx |= 0x80000000;

	return( nShiftIdx);
}




/* ------------------------------------------------------------------------- */
static int dec_search_by_contrast( YUY_T *const psLine, YUY_T *const psRefLine)
/* ------------------------------------------------------------------------- */
{
	static int		nContCor, nCurContCor, nDiff0, nDiff1, nDiff2;
	static int		nContrast, nShiftIdx, x, nMaxX;
	static u_long	nSum;
	static int		nSmallestContrast, nPos, k, j;
	static int		ntPtCont[NBR_CONTRAST];
	static int		ntDiffLum[NBR_CONTRAST];


	if ( psLine == NULL || psRefLine == NULL)
		return(NOK);

	/*	init	*/
	nShiftIdx = nSmallestContrast = -1;
	sCtx.ntContCrl[0] = sCtx.ntContCrl[1] = sCtx.ntContCrl[2] = 0;
	ntPtCont[0] = ntPtCont[1] = ntPtCont[2] = 0;
	ntDiffLum[0] = ntDiffLum[1] = ntDiffLum[2] = 0;
	nMaxX = sVP.CurrentX - ntShift[2] - sCtx.nOffsetX;

	/* search for NBR_CONTRAST max contrast in current line	*/
	for ( x = ntShift[2] + sCtx.nOffsetX; x < nMaxX; x += 2) {

		nContrast = abs( psLine[x].yX - psLine[x + 2].yX);

		if ( nContrast > nSmallestContrast) {
			nPos = 0;
			for ( j = 1 ; j < NBR_CONTRAST; j++) {
				if ( ntDiffLum[nPos] > ntDiffLum[j])
					nPos = j;
			}

			if ( ntDiffLum[nPos] < nContrast) {
				nSmallestContrast = ntDiffLum[nPos];
				ntDiffLum[nPos] = nContrast;
				ntPtCont[nPos] = x; 
			}
		}
	}


	for ( k = 0; k < NBR_CONTRAST; k++) {

		if ( ntPtCont[k] == 0)
			continue;
		
		/*	correlation arround contrast point	*/
		for ( x = ntPtCont[k] - 6, nMaxX = x + ntShift[1]; x < nMaxX; x++) { 

			nDiff0 = psRefLine[x].yX - psLine[x].yX;
			nDiff1 = psRefLine[x].yX - psLine[x + ntShift[1]].yX;
			nDiff2 = psRefLine[x].yX - psLine[x + ntShift[2]].yX;
				
			sCtx.ntContCrl[0] += nDiff0 * nDiff0;
			sCtx.ntContCrl[1] += nDiff1 * nDiff1;
			sCtx.ntContCrl[2] += nDiff2 * nDiff2;
		}
	}

	if ( sCtx.ntContCrl[0] < sCtx.ntContCrl[1])
		nShiftIdx = ( sCtx.ntContCrl[2] < sCtx.ntContCrl[0]) ? 2 : 0;
	else
		nShiftIdx = ( sCtx.ntContCrl[1] < sCtx.ntContCrl[2]) ? 1 : 2;

	/* test on validity	*/
	nSum = (sCtx.ntContCrl[0] + sCtx.ntContCrl[1] + sCtx.ntContCrl[2]);
	if ( (sCtx.ntContCrl[nShiftIdx] << 3) > nSum)
		nShiftIdx |= 0x80000000;

	
	return( nShiftIdx);

}



/* ------------------------------------------------------------------------- */
static int dec_search_by_rep( LINE_T *const psLine, LINE_T *const psRef)
/* ------------------------------------------------------------------------- */
{
	static int		nDiff0, nDiff1, nDiff2, nMaxX, nShiftIdx, x;
	static u_long	nSum;
	static int		*p, nRefShiftIdx, nRefShift;
	

	if ( psLine->psYuy == NULL || psRef->psYuy == NULL)
		return(NOK);

	nMaxX = sVP.CurrentX - sCtx.nOffsetX - ntShift[2];

	nRefShiftIdx = psRef->nShiftIdx[sCtx.nCurModeFct][sCtx.nCurModeDec] & 3;
	
	if ( nRefShiftIdx > 2)	// debug purpose
		nRefShiftIdx = 0;

	nRefShift = ntShift[nRefShiftIdx];
	p = psLine->nDiff[sCtx.nCurModeFct] + 2;

	if ( p[nRefShiftIdx] == 0) {

		for ( x = sCtx.nOffsetX; x < nMaxX; x += sCtx.sStatus.nNbrPt) {

			nDiff0 = psRef->psYuy[x + nRefShift].yX - psLine->psYuy[x].yX;
			p[nRefShiftIdx]  += nDiff0 * nDiff0;
		}
	}

	if ( p[nRefShiftIdx - 1] == 0) {

		for ( x = sCtx.nOffsetX; x < nMaxX; x += sCtx.sStatus.nNbrPt) {

			nDiff1 = psRef->psYuy[x + nRefShift].yX - psLine->psYuy[x + ntShift[1]].yX;
			p[nRefShiftIdx - 1]  += nDiff1 * nDiff1;
		}
	}
	
	if ( p[nRefShiftIdx - 2] == 0) {

		for ( x = sCtx.nOffsetX; x < nMaxX; x += sCtx.sStatus.nNbrPt) {

			nDiff2 = psRef->psYuy[x + nRefShift].yX - psLine->psYuy[x + ntShift[2]].yX;
			p[nRefShiftIdx - 2]  += nDiff2 * nDiff2;
		}
	}			

	if ( p[nRefShiftIdx] < p[nRefShiftIdx - 1])
		nShiftIdx = ( p[nRefShiftIdx - 2] < p[nRefShiftIdx]) ? 2 : 0;
	else
		nShiftIdx = ( p[nRefShiftIdx - 1] < p[nRefShiftIdx - 2]) ? 1 : 2;

	/* test on validity	*/
	nSum = ( p[nRefShiftIdx] + p[nRefShiftIdx - 1] + p[nRefShiftIdx - 2]);
	if ( (u_long)( p[nRefShiftIdx - nShiftIdx] << 2) > nSum)
		nShiftIdx |= 0x80000000;
	
	return( nShiftIdx);
}



/* ------------------------------------------------------------------------- */
static int dec_key_calc( PWORD const pwReg, const BYTE ucZ)
/* ------------------------------------------------------------------------- */
{
	static u_char	ucX, ucY;
	static int		nXYZ;

	
	if ( pwReg == NULL)
		return( 0);

	ucY = (u_char)(((*pwReg >> 2) & 1) ^ (*pwReg & 1));
	*pwReg = (u_short)((*pwReg >> 1) | (ucY << 10));
	ucX = (u_char)(*pwReg & 1);

	/* polynom + Z value	*/
	nXYZ = (ucX << 2) | (ucY << 1) | ucZ;
	/* shift index	*/
	return( ucDelayTable[nXYZ]);
}



/* ------------------------------------------------------------------------- */
int dec_key( LINE_T *const psLine)
/* ------------------------------------------------------------------------- */
{
	static int				nShiftIdx, y, nInc = 0;
	static u_short			wReg = 0;
	static u_char			ucCptZ = 0, ucZ = 0;
	static PPOLY_T			pPoly = sCtx.sKey.sPoly;
	static CHANNEL_STATE	eChannel;
	

	eChannel = sCtx.sKey.eChannel;
	/* top synchro received : init	*/
	if ( sCtx.sKey.bSync) {
		wReg = (u_short)pPoly[eChannel].nKey;
		nInc = pPoly[eChannel].nStartInc;
		ucZ = 1;
		ucCptZ = 3;
		sCtx.sKey.bSync = sCtx.sCmd.bSaveDecInfos;	//false;	// to be sure ...
	}
	
	/*	shift generation	*/
	for ( y = 0; y < sCtx.nY; y++) {
		if ( y % 287 < 1)
			continue;
		psLine[y].nShiftIdxKey = dec_key_calc( &wReg, ucZ);
	}

	/* post incrementation if necessary	*/
	nInc = nInc % 4;
	if ( nInc != 0) {
		for ( y = 0; y < nInc; y++) {
			dec_key_calc( &wReg, ucZ);
		}
	}
	nInc++;

	/* Z parameter frame count divider	*/
	ucCptZ--;
	if ( ucCptZ == 0) {
		ucZ = !ucZ;
		ucCptZ = 3;
	}
		
	return( OK);
}




/* ------------------------------------------------------------------------- */
int dec_main( LINE_T *const psLine, LINE_T *const psRef, const int y)
/* ------------------------------------------------------------------------- */
{
	static int		nShiftIdx, nShiftIdx1;
	static u_char	nDxp, nDxm;

	
	if ( psLine->psYuy == NULL || psRef->psYuy == NULL)
		return(NOK);

	nShiftIdx = nShiftIdx1 = NOK;

	/* begin line detection method 	*/
//	nShiftIdx1 = dec_search_max_start( &psLine->psYuy[sCtx.nOffsetX]);
			
	/* repartition method correlation	*/
//	nShiftIdx2 = dec_search_by_rep( psLine, psRef);

	/* begin line correlation by diff method	*/
//	nShiftIdx3 = dec_search_start_cor( &psLine->psYuy[sCtx.nOffsetX],
//										&psRef->psYuy[sCtx.nOffsetX]);

	/* contrast method correlation	*/
//	nShiftIdx4 = dec_search_by_contrast( psLine, psRefLine);

	/* Standard Azertix Method based on 9c	*/
	if ( sCtx.sCmd.eModeDec & ( DEC_AZ | DEC_AZ_START)) {
		
		if ( sCtx.sCmd.eModeDec & DEC_AZ_START) {
			sCtx.nCurModeDec = B_DEC_AZ_START;
			nShiftIdx = dec_search_max_start( &psLine->psYuy[sCtx.nOffsetX]);
		} else {
			sCtx.nCurModeDec = B_DEC_AZ;
		}
		if ( nShiftIdx == NOK) {
			nShiftIdx = ( nShiftIdx = dec_search_by_rep( psLine, psRef)) != NOK
						? nShiftIdx : 0;	// for security
		}
		/* correction	*/
		if ( (nShiftIdx & 3) == 2 && psLine->nSum3 > sCtx.nSumBlackTreshold
			&& psLine->nSum2 > psLine->nSum3 - (psLine->nSum3 >> 4))
		
			nShiftIdx = 1;
	
		if ( (nShiftIdx & 3) == 1 && psLine->nSum2 > sCtx.nSumBlackTreshold
			&& psLine->nSum1 > psLine->nSum2 - (psLine->nSum2 >> 4))
		
			nShiftIdx = 0;
		
		/* clear non validity bit for blackband	*/
		if ( nShiftIdx < 0 && ( y < sCtx.nYmin || y > sCtx.nYmax))
			nShiftIdx &= 3;

		/* count error for this method	*/
		if ( nShiftIdx < 0)
			sCtx.nErr[sCtx.nCurModeFct][sCtx.nCurModeDec]++;

		/* memo shift for the current decodage mode	*/
		psLine->nShiftIdx[sCtx.nCurModeFct][sCtx.nCurModeDec] = nShiftIdx;

	}

	/*	CriCri standard method, very similar to Azertix	*/
	if ( sCtx.sCmd.eModeDec & DEC_CR) {

		nDxm = nDxp = 0;

		sCtx.nCurModeDec = B_DEC_CR;

		nShiftIdx = ( nShiftIdx = dec_search_by_rep( psLine, psRef)) != NOK
					? nShiftIdx : 0;	// for debug

		/* correction minus	*/
		if ( nShiftIdx & 3 == 2
				&& psLine->nSum3 > ((sCtx.nSumBlackLevel << 1)
					+ (sCtx.nSumBlackLevel >> 1))
				&& psLine->nSum2 > psLine->nSum3
					- (psLine->nSum3 >> sCtx.nCorMoins))
			nDxm--;
		
		if ( (nShiftIdx & 0x3) + nDxm == 1
				&& psLine->nSum2 > ((sCtx.nSumBlackLevel << 1)
					+ (sCtx.nSumBlackLevel >> 1))
				&& psLine->nSum1 > psLine->nSum2
					- (psLine->nSum2 >> sCtx.nCorMoins))
			nDxm--;

		/* clear non validity bit for blackband	and corrections	*/
		if ( nDxp || nDxm || y < sCtx.nYmin || y > sCtx.nYmax)
			nShiftIdx &= 3;

		nShiftIdx += nDxp + nDxm;

		/* count error for this method	*/
		if ( nShiftIdx < 0)
			sCtx.nErr[sCtx.nCurModeFct][sCtx.nCurModeDec]++;

		/* memo shift for the current decodage mode	*/
		psLine->nShiftIdx[sCtx.nCurModeFct][sCtx.nCurModeDec] = nShiftIdx;
	}

	/*	CriCri Correction + method	*/
	if ( sCtx.sCmd.eModeDec & DEC_CR_CORP) {

		nDxm = nDxp = 0;

		sCtx.nCurModeDec = B_DEC_CR_CORP;
		nShiftIdx = ( nShiftIdx = dec_search_by_rep( psLine, psRef)) != NOK
					? nShiftIdx : 0;	// for debug

		/* correction minus	*/
		if ( nShiftIdx & 3 == 2
				&& psLine->nSum3 > ((sCtx.nSumBlackLevel << 1)
					+ (sCtx.nSumBlackLevel >> 1))
				&& psLine->nSum2 > psLine->nSum3
					- (psLine->nSum3 >> sCtx.nCorMoins))
			nDxm--;
		
		if ( (nShiftIdx & 3) + nDxm == 1
				&& psLine->nSum2 > ((sCtx.nSumBlackLevel << 1)
					+ (sCtx.nSumBlackLevel >> 1))
				&& psLine->nSum1 > psLine->nSum2
					- (psLine->nSum2 >> sCtx.nCorMoins))
			nDxm--;

		/* correction plus if no correction minus	*/
		if ( nDxm == 0) {

			if ( (nShiftIdx & 3) == 0
				&& psLine->nSum1 < sCtx.nSumBlackLevel
					+ ((sCtx.nSumBlackLevel << 4) >> sCtx.nCorBlackP)
				&& psLine->nSum2 > psLine->nSum1
					+ ((psLine->nSum1 << 4) >> sCtx.nCorPlus))
		
				nDxp++;
		
			if ( (nShiftIdx & 3) + nDxp == 1
				&& psLine->nSum2 < sCtx.nSumBlackLevel
					+ ((sCtx.nSumBlackLevel << 4) >> sCtx.nCorBlackP)
				&& psLine->nSum3 > psLine->nSum2
					+ ((psLine->nSum2 << 4) >> sCtx.nCorPlus))
		
				nDxp++;
		}
		
		/* clear non validity bit for blackband	and corrections	*/
		if ( nDxp || nDxm || y < sCtx.nYmin || y > sCtx.nYmax)
			nShiftIdx &= 3;

		nShiftIdx += nDxp + nDxm;

		/* count error for this method	*/
		if ( nShiftIdx < 0)
			sCtx.nErr[sCtx.nCurModeFct][sCtx.nCurModeDec]++;

		/* memo shift for the current decodage mode	*/
		psLine->nShiftIdx[sCtx.nCurModeFct][sCtx.nCurModeDec] = nShiftIdx;
	}

	/*	CriCri Correction+ & start correlation method, better than previous one	*/
	if ( sCtx.sCmd.eModeDec & DEC_CR_CORPC) {

		nDxm = nDxp = 0;

		sCtx.nCurModeDec = B_DEC_CR_CORPC;
		nShiftIdx = ( nShiftIdx = dec_search_by_rep( psLine, psRef)) != NOK
					? nShiftIdx : 0;	// for debug

		/* correction minus	*/
		if ( nShiftIdx & 3 == 2
				&& psLine->nSum3 > ((sCtx.nSumBlackLevel << 1)
					+ (sCtx.nSumBlackLevel >> 1))
				&& psLine->nSum2 > psLine->nSum3
					- (psLine->nSum3 >> sCtx.nCorMoins))
			nDxm--;
		
		if ( (nShiftIdx & 3) + nDxm == 1
				&& psLine->nSum2 > ((sCtx.nSumBlackLevel << 1)
					+ (sCtx.nSumBlackLevel >> 1))
				&& psLine->nSum1 > psLine->nSum2
					- (psLine->nSum2 >> sCtx.nCorMoins))
			nDxm--;

		/* correction plus	*/
		if ( nDxm == 0) {

			if ( (nShiftIdx & 3) == 0
				&& psLine->nSum1 < sCtx.nSumBlackLevel
					+ ((sCtx.nSumBlackLevel << 4) >> sCtx.nCorBlackP)
				&& psLine->nSum2 > psLine->nSum1
					+ ((psLine->nSum1 << 4) >> sCtx.nCorPlus))
		
				nDxp++;
		
			if ( (nShiftIdx & 3) + nDxp == 1
				&& psLine->nSum2 < sCtx.nSumBlackLevel
					+ ((sCtx.nSumBlackLevel << 4) >> sCtx.nCorBlackP)
				&& psLine->nSum3 > psLine->nSum2
					+ ((psLine->nSum2 << 4) >> sCtx.nCorPlus))
		
				nDxp++;

			/* test correction+ correlation	*/
			if ( nDxp
				&& ((dec_search_start_cor( &psLine->psYuy[sCtx.nOffsetX],
											&psRef->psYuy[sCtx.nOffsetX]) & 3)
								!= ((nShiftIdx & 3) + nDxp)))
				nDxp = 0;
		}
		
		/* clear non validity bit for blackband	and corrections	*/
		if ( nDxp || nDxm || y < sCtx.nYmin || y > sCtx.nYmax)
			nShiftIdx &= 3;

		nShiftIdx += nDxp + nDxm;

		/* count error for this method	*/
		if ( nShiftIdx < 0)
			sCtx.nErr[sCtx.nCurModeFct][sCtx.nCurModeDec]++;

		/* memo shift for the current method	*/
		psLine->nShiftIdx[sCtx.nCurModeFct][sCtx.nCurModeDec] = nShiftIdx;
	}

	return( OK);

}

